import subprocess
from datetime import datetime

from mock import ANY, patch

from locolanderweb.models import (
    STATUS_ERROR,
    STATUS_MERGED,
    MergeRequest,
    RunLog,
)
from locolanderweb.tasks import run_project, run_request, scan_project
from locolanderweb.tests.test_base import BaseTestCase


MERGE_REQUESTS = [
    dict(
        source='source1', target='target1', reviewers='a,b,c',
        date_created=datetime(2013, 4, 23), commit_message='something 1',
    ),
    dict(
        source='source2', target='target2', reviewers='pepe',
        date_created=datetime(2012, 2, 29), commit_message='something else',
    ),
    dict(
        source='source3', target='target3', reviewers='pepe y pepa',
        date_created=datetime.utcnow(), commit_message='',
    ),
]


class RunProjectTestCase(BaseTestCase):

    def setUp(self):
        super(RunProjectTestCase, self).setUp()
        patcher = patch('locolanderweb.tasks.run_request')
        self.mock_run_request = patcher.start()
        self.addCleanup(patcher.stop)

    def test_run_project(self):
        project = self.factory.make_project()
        merge_request = self.factory.make_merge_request(project=project)

        run_project(project)
        self.mock_run_request.assert_called_once_with(merge_request)

    def test_run_project_with_no_approved_requests(self):
        project = self.factory.make_project()

        run_project(project)
        self.assertFalse(self.mock_run_request.called)


class RunRequestTestCase(BaseTestCase):

    def setUp(self):
        super(RunRequestTestCase, self).setUp()
        patcher = patch('locolanderweb.tasks.subprocess.check_output')
        self.mock_check_output = patcher.start()
        self.addCleanup(patcher.stop)

    def test_run_request(self):
        self.mock_check_output.return_value = 'SUCCESS'

        merge_request = self.factory.make_merge_request()

        run_request(merge_request)

        cmd = [
            ANY,
            merge_request.project.name,
            merge_request.source,
            merge_request.target,
            merge_request.commit_message,
            merge_request.author,
            ]
        self.mock_check_output.assert_called_once_with(
            cmd, stderr=subprocess.STDOUT)
        real_cmd = self.mock_check_output.call_args[0][0][0]
        self.assertTrue(real_cmd.endswith('docker/scripts/locolander'))

        self.assertEqual(RunLog.objects.all().count(), 1)
        run_log = RunLog.objects.get(merge_request=merge_request)
        self.assertEqual(run_log.raw, 'SUCCESS')
        self.assertEqual(run_log.return_code, 0)

        # refresh MR object
        mr = MergeRequest.objects.get(pk=merge_request.pk)
        self.assertEqual(mr.status, STATUS_MERGED)

    def test_run_request_with_error(self):
        merge_request = self.factory.make_merge_request()

        cmd = [
            ANY,
            merge_request.project.name,
            merge_request.source,
            merge_request.target,
            merge_request.commit_message,
            merge_request.author,
            ]
        error = subprocess.CalledProcessError(1, cmd, output='FAILURE')
        self.mock_check_output.side_effect = error

        run_request(merge_request)

        self.mock_check_output.assert_called_once_with(
            cmd, stderr=subprocess.STDOUT)
        real_cmd = self.mock_check_output.call_args[0][0][0]
        self.assertTrue(real_cmd.endswith('docker/scripts/locolander'))

        self.assertEqual(RunLog.objects.all().count(), 1)
        run_log = RunLog.objects.get(merge_request=merge_request)
        self.assertEqual(run_log.raw, 'FAILURE')
        self.assertEqual(run_log.return_code, 1)

        # refresh MR object
        mr = MergeRequest.objects.get(pk=merge_request.pk)
        self.assertEqual(mr.status, STATUS_ERROR)


class ScanProjectTestCase(BaseTestCase):

    def test_scan_project(self):
        project = self.factory.make_project()

        service = self.get_service_from_url.return_value
        service.approved_requests_for_project_url.return_value = MERGE_REQUESTS

        scan_project(project)

        self.assertEqual(MergeRequest.objects.filter(project=project).count(),
                         len(MERGE_REQUESTS))
        for mr in MERGE_REQUESTS:
            self.assertEqual(
                MergeRequest.objects.filter(**mr).count(), 1)

    def test_scan_project_with_no_approved_request(self):
        project = self.factory.make_project()

        service = self.get_service_from_url.return_value
        service.approved_requests_for_project_url.return_value = []

        scan_project(project)

        self.assertEqual(MergeRequest.objects.filter(project=project).count(),
                         0)
