from django.core.urlresolvers import reverse
from django.utils import html
from mock import ANY, patch
from pyquery import PyQuery

from locolanderweb.forms import ProjectForm
from locolanderweb.tests.test_models import BaseTestCase


class ProjectListTestCase(BaseTestCase):

    url = reverse('project-list')

    def setUp(self):
        super(ProjectListTestCase, self).setUp()
        self.user = self.factory.make_user(password=self.password)
        assert self.client.login(username=self.user.username,
                                 password=self.password)

        self.factory.make_project(user=self.user)
        self.factory.make_project(user=self.user)
        self.factory.make_project(user=self.user)
        self.factory.make_project()  # project for another user

    def test_get(self):
        projects = self.user.project_set.all()
        assert projects.count() == 3
        response = self.client.get(self.url)

        self.assertContains(response, 'Your projects')
        html_projects = PyQuery(response.content).find('.project')
        self.assertEqual(len(html_projects), projects.count())
        projects_data = zip(html_projects.find('.project-name'),
                            html_projects.find('.project-url a'))
        for name, url in projects_data:
            self.assertEqual(
                self.user.project_set.filter(
                    name=name.text.strip(), url=url.text.strip()).count(), 1)


class ProjectAddTestCase(BaseTestCase):

    url = reverse('project-add')

    def setUp(self):
        super(ProjectAddTestCase, self).setUp()
        self.user = self.factory.make_user(password=self.password)
        assert self.client.login(username=self.user.username,
                                 password=self.password)
        self.data = dict(name='foo', url='lp:locolander')

    def test_get(self):
        assert self.user.project_set.all().count() == 0
        response = self.client.get(self.url)

        self.assertContains(response, ProjectForm(owner=self.user).as_table())
        self.assertEqual(self.user.project_set.all().count(), 0)

    def test_post(self):
        assert self.user.project_set.all().count() == 0
        response = self.client.post(self.url, data=self.data, follow=True)

        messages = [m.message for m in response.context['messages']]
        self.assertIn('Project foo added.', messages)


class ProjectDetailTestCase(BaseTestCase):

    def setUp(self):
        super(ProjectDetailTestCase, self).setUp()
        self.user = self.factory.make_user(password=self.password)
        assert self.client.login(username=self.user.username,
                                 password=self.password)

        self.project = self.factory.make_project(user=self.user)
        self.url = reverse('project-detail',
                           kwargs=dict(project_id=self.project.id))

    def test_get(self):
        response = self.client.get(self.url)

        self.assertContains(response, 'Project %s' % self.project.name)

    def test_detail_does_not_include_runlog(self):
        response = self.client.get(self.url)

        self.assertNotContains(response, '<div class="runlog-entry">')

    def test_detail_includes_runlog(self):
        mr = self.factory.make_merge_request(project=self.project)
        self.factory.make_run_log(merge_request=mr)
        self.factory.make_run_log(merge_request=mr)
        response = self.client.get(self.url)

        self.assertContains(response, '<div class="runlog-entry">', count=2)

    def test_detail_includes_land_now_button(self):
        merge_request = self.factory.make_merge_request(project=self.project)
        response = self.client.get(self.url)

        url = reverse('request-run', args=[merge_request.id])
        self.assertContains(response, '<form action="%s" method="POST">' % url)

    def test_details_includes_author(self):
        mr = self.factory.make_merge_request(project=self.project)

        response = self.client.get(self.url)

        author_html = html.escape(mr.author)
        self.assertContains(response, author_html)


class MergeRequestRunTestCase(BaseTestCase):

    def setUp(self):
        super(MergeRequestRunTestCase, self).setUp()
        self.user = self.factory.make_user(password=self.password)
        assert self.client.login(username=self.user.username,
                                 password=self.password)
        self.project = self.factory.make_project(user=self.user)
        self.merge_request = self.factory.make_merge_request(
            project=self.project)
        self.url = reverse('request-run',
                           kwargs=dict(merge_request_id=self.merge_request.id))

    def test_get(self):
        response = self.client.get(self.url)
        self.assertTemplateUsed(response, 'locolanderweb/project/detail.html')
        self.assertEqual(response.context['project'],
                         self.project)

    def test_get_not_existing(self):
        url = reverse('request-run', kwargs=dict(merge_request_id=99))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_get_non_owner(self):
        # make new user
        user = self.factory.make_user(password=self.password)
        # login using non-owner user
        self.client.login(username=user.username, password=self.password)

        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 404)

    @patch('locolanderweb.views.messages.info')
    @patch('locolanderweb.tasks.run_request.delay')
    def test_post(self, mock_delay, mock_info):
        response = self.client.post(self.url)
        mock_delay.assert_called_once_with(self.merge_request)
        msg = 'Merge of {mr.source} into {mr.target} requested.'
        mock_info.assert_called_once_with(ANY,
                                          msg.format(mr=self.merge_request))
        self.assertRedirects(
            response,
            reverse('project-detail',
                    kwargs=dict(project_id=self.project.id)))
